#!/bin/ksh
#
# Script removes installed products
# Return codes:
#   0 = OK
#   1 = no products to delete
#   2 = failed
#   3 = failed due to running processes found
#

PROGNAME=$(basename $0)

Err_report() {
   case $1 in
      warn) print -u2 "$PROGNAME warning: $2,continuing ..."
            sleep 2
            return
            ;;
      fail) print -u2 "$PROGNAME failure: $2."
            exit 2
            ;;
      info) print -u2 "$PROGNAME        : $2."
            return
            ;;
   esac
}

Use_report() {
    print -u2 "$PROGNAME [-f] [-i] [-h candle_directory] [product platformCode]"
    print -u2 "\t-f Force delete, suppressing confirmation messages and prompts."
    print -u2 "\t-i Ignore all running processes."
    print -u2 "\t'product' A two-letter code for the product to be uninstalled."
    print -u2 "\t'platformCode' Platform code (such as aix513, sol286, hp11, etc.) for the product."
    exit 2
}

# Uninstall a product and its prereqs
uninstall_product() {
  pc=$1
  platform=$2

  # Nuke CandleHome!
  if [[ "$pc" = "REMOVE" && "$platform" = "EVERYTHING" ]]
  then
    print "Removing ITM home: $CANDLEHOME\n"
    cd $originalDir
    cmdHome=$(cd $(dirname $0) ; dirname $(pwd) )
    if [[ $(pwd) = "$CANDLEHOME/bin" || "$CANDLEHOME" = "$cmdHome" ]]
    then
      Err_report warn "$CANDLEHOME/bin may have to be removed manually after this script completes"
    fi
    Err_report warn "Purging $CANDLEHOME"


    if [ -f $CANDLEHOME/registry/AutoStart ]
    then
       remove_autostart
    fi

    hp_forcewait

    rm -rf $CANDLEHOME
    exit
  fi

  # Not installed if there's no VER file
  cd $regDir
  vFile=$pc$platform.ver
  [ ! -f $vFile ] && Err_report fail "product \"$pc\" for platform \"$platform\" not installed in $CANDLEHOME"

  # Delete prereqs. Get each prereq for the product (besides ci and jr)
  for pre in $(grep "^preReq" $vFile|grep -v " ci"|grep -v " jr"|cut -f1 -d"|"|cut -f3 -d " ")
  do
    prePC=$(print $pre|cut -c1-2)
    preArch=${pre#$prePC}
    # Remove only if noone else has the same prereq
    sharedPrereq=$(grep " = $pre" $(ls *.ver|grep -v "$vFile"))
    [ -z "$sharedPrereq" ] && remove_product $prePC $preArch 
  done

  # Remove the product
  # OS agent is slow to down go, wait for it
  if [ "$pc" = "ux" ]
  then
    hp_forcewait
  fi
  remove_product $pc $platform "removed from $CANDLEHOME" 
  update_autostart $pc
}


update_autostart()
{
   if [ "$(whoami)" != "root" ]; then
      Err_report warn "You have to be a root user to update auto restart script"
   else
      if [ -f $CANDLEHOME/registry/AutoStart ]; then
	editString1="start $pc"
	editString2="stop $pc"
	filenum=`cat $CANDLEHOME/registry/AutoStart`
	CurRegPath=$CANDLEHOME/registry
	ArchTbl=$CurRegPath/archdsc.tbl
	DynShl=$CANDLEHOME/bin/dynarch.shl
	. $DynShl $ArchTbl 2>/dev/null

	os=$(print $cArch|cut -c1-2)
	[ "$os" = li ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = lx ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = lp ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = ls ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = hp ] && rcFile="/sbin/init.d/ITMAgents$filenum"
	[ "$os" = so ] && rcFile="/etc/init.d/ITMAgents$filenum"
	[ "$os" = ai ] && rcFile=/etc/rc.itm$filenum

	while true
	do
	  if [[ -n "$(grep "$editString1" $rcFile 2>/dev/null)" ]]; then
	     editString="/"$editString1"/d"
	     update_rcFile
   	     if [[ "$checkit" = "\?"* ]]; then
	      Err_report fail "Delete of agent start all record failed."
	     else
	      Err_report info "Delete of agent start all record successful."
	     fi
	  else break
	  fi
	done
	
	while true
	do
	  if [[ -n "$(grep "$editString2" $rcFile 2>/dev/null)" ]]; then
	     editString="/"$editString2"/d"
	     update_rcFile
   	     if [[ "$checkit" = "\?"* ]]; then
	      Err_report fail "Delete of agent start all record failed."
	     else
	      Err_report info "Delete of agent start all record successful."
	     fi
	  else break
	  fi
	done
      fi
   fi
}


update_rcFile()
{
checkit=$(ed -s << END
e $rcFile
$editString
w
q
END)
}




# Remove a product or a prereq (component)
remove_product() {
  product=$1
  binarch=$2
  inform=$3


  rm -rf $CANDLEHOME/$binarch/$product 2> /dev/null
  [ $? -ne 0 ] && Err_report warn "unable to remove product directory for $product"
  #if file doesn't exist don't output error message
  if [ -f $CANDLEHOME/registry/$product$binarch.ver ]
  then
    rm $CANDLEHOME/registry/$product$binarch.ver 2> /dev/null
    [ $? -ne 0 ] && Err_report warn "unable to remove version file for $product"
  fi

  name=$(grep "^$product" $regDir/proddsc.tbl|head -1|cut -f2 -d"|")
  platformDesc=$(grep "^$binarch" $regDir/archdsc.tbl|cut -f2 -d"|")
  if [ -n "$inform" ]
  then
    print "$inform ..."
    print "  \"$name\" for platform $platformDesc"
  fi

  # Log the deletion also
  logfile=$CANDLEHOME/logs/candle_installation.log
  print "uninstall.sh  : $(date)" >> $logfile
  print "  >$inform .." >> $logfile
  print "  >\"$name\" for platform $platformDesc" >> $logfile
}

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles()
{
  typeset -i maxAge=5  # Remove "remote" token files older than 5 days.
  typeset -i age=0 dot=0 doy=0
  typeset PID name
  
  ls ${baseTok}* | grep -v $thisTok > /dev/null 2>&1  # Other installs?
  [ $? -eq 0 ] || return
  
  doy=$(date +"%j")  # Day of the year (001 - 366)
  
  # Remove token files:
  # 1) created on this machine that are no longer connected to any process.
  # 2) created on other machines that are older than maxAge.
  
  for name in $(ls ${baseTok}* | grep -v $thisTok)
  do
    cat $name | grep "$thisMach" > /dev/null 2>&1 
    if [ $? -eq 0 ]  # The token file was created on this machine.
    then
      PID=${name##*_}
      ps -p $PID | grep $PID > /dev/null 2>&1
      [ $? -ne 0 ] && rm -f $name
    else  
      cat $name | grep "|" > /dev/null 2>&1  # Ensure the day of year field exists.
      [ $? -eq 0 ] || continue
      
      dot=$(cat $name | cut -d"|" -f2 | cut -d"|" -f1)  # Day the token file was created (001 - 366).

      if [ doy -ge dot ]
      then
        age=doy-dot
      else  # This could happen towards the end of a year, so "normalize" the day of the year.
        doy=366+doy
        age=doy-dot
      fi

      [ age -gt maxAge ] && rm -f $name
    fi
  done
  
  # Warn the user about possible simultaneous installs.
  
  ls ${baseTok}* | grep -v $thisTok > /dev/null 2>&1  # Check for other installs.
  if [ $? -eq 0 ] 
  then
    print "\nAn install may currently be running in \"$CANDLEHOME\" from"
    print "the following machine(s): \n"
    for name in $(ls ${baseTok}* | grep -v $thisTok)
    do
      print "  \c"; cat $name
    done
  
    print "\nContinue with this uninstallation [ y or n; \"n\" is default ]?  \c"
    read
    case $REPLY in
      y*|Y*) ;;
    
          *) rm -f $thisTok
             exit
             ;;
    esac
  fi
}
# Check for running processes before uninstall

checkProcesses()
{
  typeset pFile=$CANDLEHOME/registry/proddsc.tbl
  typeset rFile=$CANDLEHOME/config/.ConfigData/RunInfo
  typeset PID desc fieldSep fld23 msg pc proc 
  typeset thisMachShort 

  # No RunInfo means nothing's running
  [ ! -f $rFile ] && return
  
  # Determine whether it's BigK or dinkySQL.
  
  fieldSep=$(grep "FIELDSEP" $rFile | cut -d "=" -f2 2> /dev/null)
  
  # See if there are any agents/CMS's running from this <candlehome> on this machine.
  
  if [ -n "$fieldSep" ]  # BigK.
  then
    # EJF (#11545) make the check for the hostname in the RunInfo file a bit smarter
    
    # check if thisMach contains a fully qualifed hostname
    print $thisMach | grep "\." > /dev/null 2>&1
    
    # if thisMach does contain a fully qualified hostname
    if [[ $? -eq 0 ]]
    then
      # search for the fully qualified name in RunInfo
      cat $rFile | grep -i "^$thisMach$fieldSep" > /dev/null 2>&1
      
      # if the fully qualified name was found in RunInfo
      if [[ $? -eq 0 ]]
      then
        # record the match
        fld23=$(cat $rFile | grep -i "^$thisMach$fieldSep" | cut -d "$fieldSep" -f2,3)  # fld23 -> ms|35594
      
      # else search RunInfo for the short name
      else
        # make the short name and search RunInfo for it
        thisMachShort=$(print $thisMach | cut -d "." -f1)
        cat $rFile | grep -i "^$thisMachShort$fieldSep" > /dev/null 2>&1
        
        # if the short name was found in RunInfo
        if [[ $? -eq 0 ]]
        then
          # record the match
          fld23=$(cat $rFile | grep -i "^$thisMachShort$fieldSep" | cut -d "$fieldSep" -f2,3)  # fld23 -> ms|35594
          
        else
          # neither the full nor short name was found in RunInfo so return
          return
        fi
      fi
    
    # else thisMach contains a short hostname
    else
      # search RunInfo for the short name
      cat $rFile | grep -i "^$thisMach[$fieldSep\.]" > /dev/null 2>&1
      
      # if the short name was found in RunInfo
      if [[ $? -eq 0 ]]
      then
        # record the match
        fld23=$(cat $rFile | grep -i "^$thisMach[$fieldSep\.]" | cut -d "$fieldSep" -f2,3)  # fld23 -> ms|35594
          
      else
        # neither the full nor short name was found in RunInfo so return
        return
      fi
    fi
    
  else  # dinkySQL.
    cat $rFile | sed "s/  */ /g" | grep "$thisMach" | cut -c1 | grep "N" > /dev/null 2>&1
    [ $? -eq 0 ] || return
    fld23=$(cat $rFile | sed "s/  */ /g" | grep "$thisMach" | grep "^N" | cut -d" " -f2)  # fld23 -> ms35594
  fi
  
  # Check to see if the processes are actually running.
  
  for proc in $fld23
  do
    [ -n "$fieldSep" ] && PID=$(print "$proc" | cut -d "$fieldSep" -f2) || PID=$(print $proc | cut -c3-)
    ps -p $PID | grep $PID > /dev/null 2>&1
    [ $? -eq 0 ] || continue
    msg="\\nThe following processes must be stopped before attempting uninstall: \\n"
    break
  done
  
  if [ -n "$msg" ]
  then
    print "$msg"
    for proc in $fld23
    do
      [ -n "$fieldSep" ] && pc=$(print "$proc" | cut -d "$fieldSep" -f1) || pc=$(print $proc | cut -c1-2)
    
      # Get the product description from the proddsc.tbl file.  If there are running processes,
      # then this file should exist.
    
      if [ -r $pFile ]  
      then
        desc=$(grep "^$pc|" $pFile | head -1 | cut -d"|" -f2)  # desc -> "NetView/AIX Alert Adapter"
        [ -n "$desc" ] || desc=$pc  # Just in case.
      fi
    
      [ -n "$fieldSep" ] && PID=$(print "$proc" | cut -d "$fieldSep" -f2) || PID=$(print $proc | cut -c3-)
    
      print "  Product = $desc  PID = $PID"
    done
    print
    exit 3

  fi
}
hp_forcewait()
{

#  OS agent is not cleaning up its children so we have to wait on HP for 
#  them to die

    CurRegPath=$CANDLEHOME/registry
    ArchTbl=$CurRegPath/archdsc.tbl
    DynShl=$CANDLEHOME/bin/dynarch.shl


    . $DynShl $ArchTbl 2>/dev/null
    os=$(print $cArch|cut -c1-2)
 
    if [ "$os" = hp ]
    then
       print Waiting for agent to shutdown
       sleep 70
    fi

    return

}
remove_autostart()
{

    CurRegPath=$CANDLEHOME/registry
    ArchTbl=$CurRegPath/archdsc.tbl
    DynShl=$CANDLEHOME/bin/dynarch.shl


    . $DynShl $ArchTbl 2>/dev/null

    os=$(print $cArch|cut -c1-2)
    cat $CANDLEHOME/registry/AutoStart | read filenum
    filenum=`cat $CANDLEHOME/registry/AutoStart`

  # If we're already root, there's no need to su

  if [ "$(whoami)" = "root" ]
  then


    if [ "$os" = li ]
    then
       rm -f  /etc/rc0.d/K10ITMAgents$filenum
       rm -f  /etc/rc1.d/K10ITMAgents$filenum
       rm -f  /etc/rc2.d/S99ITMAgents$filenum
       rm -f  /etc/rc3.d/S99ITMAgents$filenum
       rm -f  /etc/rc4.d/S99ITMAgents$filenum
       rm -f  /etc/rc5.d/S99ITMAgents$filenum
       rm -f  /etc/rc6.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc0.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc1.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc2.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc3.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc4.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc5.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc6.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/ITMAgents$filenum
       return
    fi
 
    if [ "$os" = ls ]
    then
       rm -f  /etc/rc0.d/K10ITMAgents$filenum
       rm -f  /etc/rc1.d/K10ITMAgents$filenum
       rm -f  /etc/rc2.d/S99ITMAgents$filenum
       rm -f  /etc/rc3.d/S99ITMAgents$filenum
       rm -f  /etc/rc4.d/S99ITMAgents$filenum
       rm -f  /etc/rc5.d/S99ITMAgents$filenum
       rm -f  /etc/rc6.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc0.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc1.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/rc2.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc3.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc4.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc5.d/S99ITMAgents$filenum
       rm -f  /etc/init.d/rc6.d/K10ITMAgents$filenum
       rm -f  /etc/init.d/ITMAgents$filenum
       return
    fi
 
    if [ "$os" = hp ]
    then
       rm -f  /sbin/rc0.d/K100ITMAgents$filenum
       rm -f  /sbin/rc1.d/K100ITMAgents$filenum
       rm -f  /sbin/rc2.d/S500ITMAgents$filenum
       rm -f  /sbin/init.d/ITMAgents$filenum
       return
    fi

    if [ "$os" = so ]
    then 
       rm -f /etc/rc0.d/K10ITMAgents$filenum
       rm -f /etc/rc1.d/K10ITMAgents$filenum
       rm -f /etc/rc2.d/S99ITMAgents$filenum
       rm -f /etc/init.d/ITMAgents$filenum
       return
    fi
 
    if [ "$os" = ai ]
    then 
       rmitab rcitm$filenum
       rm -f /etc/rc.itm$filenum
       return
    fi

  else
     print "\nroot user access is required to remove automatic restart files"
 
     # If we're not root and we're in force mode, bail out
     if [ "$force" = "yes" ]
     then
        return
     fi

     if [ "$os" = li ]
     then
        cmd1="rm -f  /etc/rc0.d/K10ITMAgents$filenum"
        cmd2="rm -f  /etc/rc1.d/K10ITMAgents$filenum"
        cmd3="rm -f  /etc/rc2.d/S99ITMAgents$filenum"
        cmd4="rm -f  /etc/rc3.d/S99ITMAgents$filenum"
        cmd5="rm -f  /etc/rc4.d/S99ITMAgents$filenum"
        cmd6="rm -f  /etc/rc5.d/S99ITMAgents$filenum"
        cmd7="rm -f  /etc/rc6.d/K10ITMAgents$filenum"
        cmd8="rm -f  /etc/init.d/rc0.d/K10ITMAgents$filenum"
        cmd9="rm -f  /etc/init.d/rc1.d/K10ITMAgents$filenum"
        cmd10="rm -f  /etc/init.d/rc2.d/S99ITMAgents$filenum"
        cmd11="rm -f  /etc/init.d/rc3.d/S99ITMAgents$filenum"
        cmd12="rm -f  /etc/init.d/rc4.d/S99ITMAgents$filenum"
        cmd13="rm -f  /etc/init.d/rc5.d/S99ITMAgents$filenum"
        cmd14="rm -f  /etc/init.d/rc6.d/K10ITMAgents$filenum"
        cmd15="rm -f  /etc/init.d/ITMAgents$filenum"
        cmd="$cmd1;$cmd2;$cmd3;$cmd4;$cmd5;$cmd6;$cmd7;$cmd8;$cmd9;$cmd10;$cmd11;$cmd12;$cmd13;$cmd14;$cmd15"
     fi

     if [ "$os" = ls ]
     then
        cmd1="rm -f  /etc/rc0.d/K10ITMAgents$filenum"
        cmd2="rm -f  /etc/rc1.d/K10ITMAgents$filenum"
        cmd3="rm -f  /etc/rc2.d/S99ITMAgents$filenum"
        cmd4="rm -f  /etc/rc3.d/S99ITMAgents$filenum"
        cmd5="rm -f  /etc/rc4.d/S99ITMAgents$filenum"
        cmd6="rm -f  /etc/rc5.d/S99ITMAgents$filenum"
        cmd7="rm -f  /etc/rc6.d/K10ITMAgents$filenum"
        cmd8="rm -f  /etc/init.d/rc0.d/K10ITMAgents$filenum"
        cmd9="rm -f  /etc/init.d/rc1.d/K10ITMAgents$filenum"
        cmd10="rm -f  /etc/init.d/rc2.d/S99ITMAgents$filenum"
        cmd11="rm -f  /etc/init.d/rc3.d/S99ITMAgents$filenum"
        cmd12="rm -f  /etc/init.d/rc4.d/S99ITMAgents$filenum"
        cmd13="rm -f  /etc/init.d/rc5.d/S99ITMAgents$filenum"
        cmd14="rm -f  /etc/init.d/rc6.d/K10ITMAgents$filenum"
        cmd15="rm -f  /etc/init.d/ITMAgents$filenum"
    	cmd="$cmd1;$cmd2;$cmd3;$cmd4;$cmd5;$cmd6;$cmd7;$cmd8;$cmd9;$cmd10;$cmd11;$cmd12;$cmd13;$cmd14;$cmd15"
     fi

    if [ "$os" = hp ]
    then
       cmd1="rm -f  /sbin/rc0.d/K100ITMAgents$filenum"
       cmd2="rm -f  /sbin/rc1.d/K100ITMAgents$filenum"
       cmd3="rm -f  /sbin/rc2.d/S500ITMAgents$filenum"
       cmd4="rm -f  /sbin/init.d/ITMAgents$filenum"
       cmd="$cmd1;$cmd2;$cmd3;$cmd4"
    fi

    if [ "$os" = so ]
    then 
       cmd1="rm -f /etc/rc0.d/K10ITMAgents$filenum"
       cmd2="rm -f /etc/rc1.d/K10ITMAgents$filenum"
       cmd3="rm -f /etc/rc2.d/S99ITMAgents$filenum"
       cmd4="rm -f /etc/init.d/ITMAgents$filenum"
       cmd="$cmd1;$cmd2;$cmd3;$cmd4"
    fi
 
    if [ "$os" = ai ]
    then 
       cmd1="rmitab rcitm$filenum"
       cmd2="rm -f /etc/rc.itm$filenum"
       cmd="$cmd1;$cmd2"
    fi

    loop=yes

    while [ $loop = "yes" ]
    do
       su root -c "$cmd"
       rc=$?
       if [ $rc -ne 0 ]  # su failed
       then
          print "\nIgnore [ i ] or Try again [ y , n; \"n\" is default ]?  \c"
          read 
          case $REPLY in
            y*|Y*) 
              continue
            ;;
            i*|I*) 
              loop=no
              return
            ;;
            *) 
              print "Failing, Commands to be executed were:"
              print "$cmd"
              loop=no
            ;;
         esac
      fi
    return
    done
  fi

}

uninstall_process()
{

	  # This returns JAVA_HOME to us on the command line - set to variable and use
	  export JAVA_HOME=`$CANDLEHOME/bin/CandleGetJavaHome`
	  #TEST print "JAVA_HOME var: $JAVA_HOME"

	  TEMP_pc=$1
      TEMP_platform=$2

	  # If exists
      if [ -f $CANDLEHOME/bin/uninstall-process.sh ]
      then

		# and is executable
        if [ -x $CANDLEHOME/bin/uninstall-process.sh ]
        then

	    	print ""
			print "Processing pre-uninstall operations..."
    
			#TEST print "$CANDLEHOME/bin/uninstall-process.sh $JAVA_HOME $CANDLEHOME $TEMP_pc preuninstall $TEMP_platform uninstall"
			$CANDLEHOME/bin/uninstall-process.sh $JAVA_HOME $CANDLEHOME $TEMP_pc "preuninstall" $TEMP_platform "uninstall"

			print "Processing pre-uninstall complete."
			print ""
      
		else

			#TEST print "OK, the permissions are off, lets try and set execute"
			#TEST print "chmod a+x $CANDLEHOME/bin/uninstall-process.sh"
			chmod a+x $CANDLEHOME/bin/uninstall-process.sh 2>/dev/null

			# Now check again
			if [ -x $CANDLEHOME/bin/uninstall-process.sh ]
			then

				# We fixed the problem, good

				print ""
				print "Processing pre-uninstall operations..."
    
				#TESTING print "$CANDLEHOME/bin/uninstall-process.sh $JAVA_HOME $CANDLEHOME $TEMP_pc preuninstall $TEMP_platform uninstall"
				$CANDLEHOME/bin/uninstall-process.sh $JAVA_HOME $CANDLEHOME $TEMP_pc "preuninstall" $TEMP_platform "uninstall"

				print "Processing pre-uninstall complete."
				print ""

			else
			
				# OK, we tried once, now we tell the user for more advanced steps
			
				print ""
				print "The file [$CANDLEHOME/bin/uninstall-process.sh] exists, but is not executable by this user. Please change"
				print "the permission of this file to be executable by this user."
				print ""

				# This is a bold step to exit the entire unintsaller, but warranted since they need to try and fix this,
				# and it's easy enough to re-enter the tool. Plus, the user might not see the message

				exit 1
				
			fi

		fi

      else
		# The file does not exist, which will often happen - we silently skip this
		print ""
      fi

}

###################################################################
#                         Main Routine                            #
###################################################################

export PATH=/usr/bin:/usr/sbin:/usr/ucb:$PATH
Mode=""
Menu="yes"
force="no"
ignore="no"
[ -z "$(whoami 2>/dev/null)" ] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'

typeset -i ver=0 vr=0

typeset baseTok thisMach thisOS thisRec thisTok thisUser  # Variables used for the token file.

while getopts ":h:fi" OPTS
do case $OPTS in
   h) CANDLEHOME=$OPTARG
      ;;
   f) force="yes"
      ;;
   i) ignore="yes"
      ;;
   ?) Use_report
      ;;
   esac
done

shift $((OPTIND-1))

# Allow only product/platform combo or nothing
if [ $# -ge 2 ]
then
   pc="$1"; shift; platform="$1"
   Menu="no"
elif [ $# -eq 1 ]
then
   Use_report
else
   Menu="yes"
fi

[ -n "$CANDLEHOME" ] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
[ -d "$CANDLEHOME" ] || Err_report fail "specified ITM home is not a valid directory: $CANDLEHOME"
export CANDLEHOME
originalDir=$(pwd)

if [ $(print "$CANDLEHOME" | wc -w) -ne 1 ]
then
  Err_report fail "Blanks are not permitted in CANDLEHOME"
fi

export CANDLETMP=$CANDLEHOME/tmp

# LL - start token file stuff.

baseTok=$CANDLETMP/candle_install_

thisOS=$(uname -a | cut -d" " -f1)  # thisOS -> AIX
thisUser=$(whoami)

# Create the token file.

thisMach=$(uname -a | cut -d" " -f2)  # thisMach -> vulcan
thisTok=$baseTok${thisMach}_$$
touch $thisTok > /dev/null 2>&1
[ $? -eq 0 ] || Err_report fail "could not create token file"

# Build the description record, and add it to the token file.  This record will look like: 
# "AIX vulcan billy 17440 Friday Jun 08 14:21 2001 |159|".  The number between pipes (|)
# is the day of the year the file was created.

thisRec="$thisOS $thisMach $thisUser $$ $(date +"%A %B %d %H:%M %Y |%j|")"
print "$thisRec" > $thisTok

# Delete old token files, and warn if there is another install running in this <candlehome>.

procTokFiles 

# LL - end token file stuff.

if [ "$ignore" = "no" ]
then
  # Make sure nothing's running in this CandleHome
  checkProcesses
fi

# Get installer version and release number
ciVFile=cienv.ver
typeset curRel="" curVer="" regFile=""

regFile=$CANDLEHOME/registry/$ciVFile
[ -r $regFile ] || Err_report fail "could not read file: $regFile"

curVrmf=$(grep "^VRMF =" $regFile | cut -d"=" -f2)
if [ -z "$curVrmf" ]
then
    curVer=$(grep "^ver =" $regFile | cut -d"=" -f2)
    [ -n "$curVer" ] || Err_report fail "could not determine the version number from $regFile"
    curRel=$(grep "^rel =" $regFile | cut -d"=" -f2)
    [ -n "$curRel" ] || Err_report fail "could not determine the release number from $regFile"
else
    curVer1=$(print $curVrmf| cut -c 1,2)
    curVer2=$(print $curVrmf| cut -c 3,4)
    curVer3=$(print $curVrmf| cut -c 5,6)
    curVer4=$(print $curVrmf| cut -c 7,8)
fi

# Show banner
print " " 
print "*********** $(date) ******************"
if [[ "$(uname -s)" != "NONSTOP_KERNEL" ]]
then
  print "User      : $(whoami)\t Group: $(groups)"
  if [ -z "$curVrmf" ]
  then
    print "Host name : $(hostname)\t Installer Lvl:$curVer /$curRel" 
  else
    print "Host name : $(hostname)\t Installer Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
  fi
else
  print "User      : $(whoami)\t "
  if [ -z "$curVrmf" ]
  then
    print "Installer Lvl:$curVer /$curRel" 
  else
    print " Installer Lvl:$curVer1.$curVer2.$curVer3.$curVer4"
  fi
fi		# end uname -s
print "ITM home  : $CANDLEHOME" 
print "***********************************************************" 

infoTmp=$CANDLEHOME/tmp/info.tmp_$$
infoTmp2=$CANDLEHOME/tmp/info.tmp2_$$
regDir=$CANDLEHOME/registry

# User entered a product code / platform combo in the args
if [ "$Menu" = "no" ]
then

  # Call the uninstall-process handler
  uninstall_process $pc $platform

  uninstall_product $pc $platform

  # cm12160 - for silent install, need to remove top level directory
  # if no more products are installed and the force flag (-f) is specified
  grepCmd=grep
  [ -f /usr/xpg4/bin/grep ] && grepCmd=/usr/xpg4/bin/grep
  remaining=$(find *.ver ! -exec $grepCmd -q 'preReqALL = none' {} \; -print)

  if [[ -z "$remaining" && "$force" = "yes" ]]; then
      print "Removing ITM home: $CANDLEHOME\n"
      
      # remove the agent autostart files
      if [ -f $CANDLEHOME/registry/AutoStart ]
      then
         remove_autostart
      fi
    
      cd $originalDir
 
      hp_forcewait
 
      rm -rf $CANDLEHOME
  fi

# User entered nothing on command-line so we show the list of installed prods
else
  cd $regDir
  rm -f $infoTmp2
  Response=""
  delPC=""
  listOfOs="aix hp sol l t"
  while true
  do
    # Get a list of installed product version files. Show only version files that have prereqs
    grepCmd=grep
    [ -f /usr/xpg4/bin/grep ] && grepCmd=/usr/xpg4/bin/grep
    for pc in $(find *.ver ! -exec $grepCmd -q 'preReqALL = none' {} \; -print)
    do
      pc=$(print $pc|cut -c 1-2)
      print $pc >> $infoTmp2
    done

    if [ -z "$Response" ] && [ -f $infoTmp2 ]
    then
      print "\n...Products available to uninstall\n"
      print "Num\tProduct [ Code  Platform Version:Release Description ]"
    fi

    # Get a list of installed product version files. Show only version files that have prereqs
    grepCmd=grep
    [ -f /usr/xpg4/bin/grep ] && grepCmd=/usr/xpg4/bin/grep
    for pc in $(find *.ver ! -exec $grepCmd -q 'preReqALL = none' {} \; -print)
    do
      pc=$(print $pc|cut -c 1-2)
      print $pc >> $infoTmp2
    done

    # No products to list ... ask about removing CANDLEHOME
    if [ ! -f $infoTmp2 ]
    then
      print
      print "No products found to uninstall. Preparing to remove ITM home ..."
      if [ "$force" = "no" ]
      then
        print -n "Confirm: CANDLEHOME=$CANDLEHOME  ... OK to delete? [y/n]: "
          read answer
        else
          answer="y"
      fi
      case "$answer" in
        y|Y|yes|YES) print "Removing ITM home: $CANDLEHOME\n"
	cd $originalDir
        cmdHome=$(cd $(dirname $0) ; dirname $(pwd) )
	if [[ $(pwd) = "$CANDLEHOME/bin" || "$CANDLEHOME" = "$cmdHome" ]]
	then
	  Err_report warn "$CANDLEHOME/bin may have to be removed manually after this script completes"
	fi
        Err_report warn "Purging $CANDLEHOME"
        if [ -f $CANDLEHOME/registry/AutoStart ]
        then
           remove_autostart
        fi
 
        hp_forcewait
 
        rm -rf $CANDLEHOME
        ;;
      esac
      exit 1
    fi

    sort -u -o $infoTmp $infoTmp2
    typeset -i index=0
    while read pc
    do
      name=$(grep "^$pc" $regDir/proddsc.tbl| head -1 |cut -f2 -d"|")
      for OS in $listOfOs
      do
        if [ -n "$(ls ${pc}$OS*.ver 2>/dev/null)" ]
        then
          for vFile in $(ls ${pc}$OS*.ver)
          do
	    index=$index+1
	    if [ "$Response" = "" ]
	    then
              platform=${vFile#$pc}
              platform=${platform%.ver}
	      vrmf=$(grep "^VRMF =" $vFile|cut -f3 -d" ")
	      if [ -z "$vrmf" ]
	      then
              	version=$(grep "^ver =" $vFile|cut -f3 -d" ")
              	release=$(grep "^rel =" $vFile|cut -f3 -d" ")
              	print "$index\t$pc  $platform\t v$version:r$release  $name"
	      else
		vrmf1=$(print $vrmf| cut -c 1,2)
		vrmf2=$(print $vrmf| cut -c 3,4)
		vrmf3=$(print $vrmf| cut -c 5,6)
		vrmf4=$(print $vrmf| cut -c 7,8)
		print "$index\t$pc  $platform\t v$vrmf1.$vrmf2.$vrmf3.$vrmf4 $name"
	      fi
	    elif [ "$Response" = $index ]
	    then
              platform=${vFile#$pc}
              platform=${platform%.ver}
	      vrmf=$(grep "^VRMF =" $vFile|cut -f3 -d" ")
	      if [ -z "$vrmf" ]
	      then
              	version=$(grep "^ver =" $vFile|cut -f3 -d" ")
              	release=$(grep "^rel =" $vFile|cut -f3 -d" ")
	      	delVersion=$version
	      	delRelease=$release
	      else
		vrmf1=$(print $vrmf| cut -c 1,2)
		vrmf2=$(print $vrmf| cut -c 3,4)
		vrmf3=$(print $vrmf| cut -c 5,6)
		vrmf4=$(print $vrmf| cut -c 7,8)
		delVersion=$vrmf1.$vrmf2.$vrmf3.$vrmf4
		delRelease=""
	      fi
	      delPC=$pc
	      delPlatform=$platform
	      delName=$name
	      break
	    fi
          done
        fi
      done
    done < $infoTmp
    rm -f $infoTmp $infoTmp2

    # List displayed and user responded ... now process response

    # User entered something
    if [ "$Response" != "" ]
    then
      # User picked a valid product to uninstall
      if [ "$delPC" != "" ]
      then
	if [ "$force" = "no" ]
	then
	  if [ -n "$delRelease" ]
	  then
            print -n "Confirm: $delPC  $delPlatform  v$delVersion:r$delRelease $delName ... OK to delete? [y/n]: "
	  else
            print -n "Confirm: $delPC  $delPlatform  v$delVersion $delName ... OK to delete? [y/n]: "
	  fi
          read answer
	else
	  answer="y"
	fi
        case "$answer" in
	  y|Y|yes|YES) print uninstall_product $delPC $delPlatform

	  # Call the uninstall-process handler
	  uninstall_process $delPC $delPlatform

	  uninstall_product $delPC $delPlatform
	  delPC=""
             ;;
        esac
      else
	print "$Response was not a valid selection, please try again"
      fi
      Response=""

    # User entered nothing, or first time thru .. just prompt for a selection (or exit)
    else
      print -n "\nEnter number for a product to uninstall or \"EXIT\" to exit: "
      read Response
      case "$Response" in
        x|X|exit|EXIT) exit
           ;;
      esac
    fi
  done
fi

exit
